Friend Class ComboLBoxCustom
    Inherits System.Windows.Forms.NativeWindow

    Public Event MouseEnter As EventHandler
    Public Event MouseLeave As EventHandler
    Public Event MouseMove As MouseEventHandler

    Private _clientRectangle As Rectangle=rectangle.Empty
    Private _underMouse As Boolean = False

    Public Sub New(ByVal handle As IntPtr)
        MyBase.New()
        Me.AssignHandle(handle)
    End Sub

    Protected Overrides Sub WndProc(ByRef m As System.Windows.Forms.Message)
        Select Case m.Msg
            Case WM_SIZE
                GetClientRectangle()
            Case WM_SHOWWINDOW
                Dim hidden As Boolean = m.WParam.Equals(IntPtr.Zero)
                If _underMouse AndAlso hidden Then
                    _underMouse = False
                    OnMouseLeave(EventArgs.Empty)
                End If
            Case WM_MOUSEMOVE
                Dim position As Point = GetPoint(m.LParam)
                If HitTest(position) Xor _underMouse Then
                    _underMouse = Not _underMouse
                    If _underMouse Then
                        OnMouseEnter(EventArgs.Empty)
                    Else
                        OnMouseLeave(EventArgs.Empty)
                    End If
                Else
                    Dim button As MouseButtons = GetButtons(m.WParam)
                    Dim mea As MouseEventArgs = New MouseEventArgs(button, 0, position.X, position.Y, 0)
                    OnMouseMove(mea)
                End If
        End Select
        MyBase.WndProc(m)
    End Sub

    Protected Sub OnMouseEnter(ByVal e As EventArgs)
        RaiseEvent MouseEnter(Me, e)
    End Sub

    Protected Sub OnMouseLeave(ByVal e As EventArgs)
        RaiseEvent MouseLeave(Me, e)
    End Sub

    Protected Sub OnMouseMove(ByVal e As System.Windows.Forms.MouseEventArgs)
        RaiseEvent MouseMove(Me, e)
    End Sub

    Private Sub GetClientRectangle()
        Dim rc As RECT
        GetClientRect(Me.Handle, rc)
        _clientRectangle = New Rectangle(rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top)
    End Sub

    Private Function GetButtons(ByVal wParam As IntPtr) As MouseButtons
        Dim values As Integer = wParam.ToInt32()
        Dim button As MouseButtons = MouseButtons.None
        If (values And MK_LBUTTON) <> 0 Then button = button Or MouseButtons.Left
        If (values And MK_RBUTTON) <> 0 Then button = button Or MouseButtons.Right
        If (values And MK_MBUTTON) <> 0 Then button = button Or MouseButtons.Middle
        If (values And MK_XBUTTON1) <> 0 Then button = button Or MouseButtons.XButton1
        If (values And MK_XBUTTON2) <> 0 Then button = button Or MouseButtons.XButton2
        Return button
    End Function

    Private Function GetPoint(ByVal lParam As IntPtr) As Point
        Return New Point(lParam.ToInt32())
    End Function

    Private Function HitTest(ByVal position As Point) As Boolean
        Return _clientRectangle.Contains(position)
    End Function
End Class
