Friend Class ListToolTip
    Implements System.IDisposable

    Private _toolTip As NativeWindow
    Private _idHandle As IntPtr
    Private _activate As Boolean
    Private _automaticDelay As Integer

    Public Sub New(ByVal handle As IntPtr)
        _idHandle = handle
        CreateToolTip()
        Activate = False
        MaxWidth = 1000
        ToolTipAddTool()
    End Sub

    Public Sub Dispose() Implements System.IDisposable.Dispose
        DestroyToolTip()
        GC.SuppressFinalize(Me)
    End Sub

    Public Property Activate() As Boolean
        Get
            Return _activate
        End Get
        Set(ByVal Value As Boolean)
            _activate = Value
            ToolTipActivate(_activate)
        End Set
    End Property

    Public Property AutomaticDelay() As Integer
        Get
            Return _automaticDelay
        End Get
        Set(ByVal Value As Integer)
            _automaticDelay = Value
            ToolTipSetDelayTime(TTDT_AUTOMATIC, Value)
        End Set
    End Property

    Public Property AutoPopDelay() As Integer
        Get
            Return ToolTipGetDelayTime(TTDT_AUTOPOP)
        End Get
        Set(ByVal Value As Integer)
            ToolTipSetDelayTime(TTDT_AUTOPOP, Value)
        End Set
    End Property

    Public Property InitialDelay() As Integer
        Get
            Return ToolTipGetDelayTime(TTDT_INITIAL)
        End Get
        Set(ByVal Value As Integer)
            ToolTipSetDelayTime(TTDT_INITIAL, Value)
        End Set
    End Property

    Public Property ReshowDelay() As Integer
        Get
            Return ToolTipGetDelayTime(TTDT_RESHOW)
        End Get
        Set(ByVal Value As Integer)
            ToolTipSetDelayTime(TTDT_RESHOW, Value)
        End Set
    End Property

    Public Property Text() As String
        Get
            Dim ti As TOOLINFO = ToolTipGetToolInfo()
            Return ti.lpszText
        End Get
        Set(ByVal Value As String)
            ToolTipUpdateTipText(Value)
        End Set
    End Property

    Public Property MaxWidth() As Integer
        Get
            Return ToolTipGetMaxTipWidth()
        End Get
        Set(ByVal Value As Integer)
            ToolTipSetMaxTipWidth(Value)
        End Set
    End Property

    Protected Overrides Sub Finalize()
        DestroyToolTip()
    End Sub

    Private Sub CreateToolTip()
        _toolTip = New NativeWindow
        _toolTip.CreateHandle(ToolTipCreateParams())
    End Sub

    Private Sub DestroyToolTip()
        If Not _toolTip Is Nothing Then
            _toolTip.DestroyHandle()
            _toolTip = Nothing
        End If
    End Sub

    Private Function ToolTipCreateParams() As CreateParams
        Dim cp As CreateParams = New CreateParams
        cp.ClassName = TOOLTIPS_CLASS
        cp.ExStyle = WS_EX_TOPMOST
        cp.Style = TTS_ALWAYSTIP
        Return cp
    End Function

    Private Sub ToolTipActivate(ByVal activate As Boolean)
        SendMessage(_toolTip.Handle, TTM_ACTIVATE, activate, 0)
    End Sub

    Private Sub ToolTipSetDelayTime(ByVal dwDuration As Integer, ByVal iTime As Integer)
        SendMessage(_toolTip.Handle, TTM_SETDELAYTIME, dwDuration, MAKELONG(iTime, 0))
    End Sub

    Private Sub ToolTipAddTool()
        Dim ti As TOOLINFO = ToolInfoTemplate()
        ti.uFlags = TTF_SUBCLASS Or TTF_IDISHWND
        SendMessage(_toolTip.Handle, TTM_ADDTOOL, 0, ti)
    End Sub

    Private Function ToolTipGetToolInfo() As TOOLINFO
        Dim ti As TOOLINFO = ToolInfoTemplate()
        SendMessage(_toolTip.Handle, TTM_GETTOOLINFO, 0, ti)
        Return ti
    End Function

    Private Sub ToolTipUpdateTipText(ByVal text As String)
        Dim ti As TOOLINFO = ToolTipGetToolInfo()
        ti.lpszText = text
        SendMessage(_toolTip.Handle, TTM_UPDATETIPTEXT, 0, ti)
    End Sub

    Private Function ToolTipGetDelayTime(ByVal dwDuration As Integer) As Integer
        Dim milliSeconds As IntPtr
        milliSeconds = SendMessage(_toolTip.Handle, TTM_GETDELAYTIME, dwDuration, 0)
        Return milliSeconds.ToInt32()
    End Function

    Public Sub ToolTipSetMaxTipWidth(ByVal iWidth As Integer)
        SendMessage(_toolTip.Handle, TTM_SETMAXTIPWIDTH, 0, iWidth)
    End Sub

    Public Function ToolTipGetMaxTipWidth() As Integer
        Dim iWidth As IntPtr
        iWidth = SendMessage(_toolTip.Handle, TTM_GETMAXTIPWIDTH, 0, 0)
        Return iWidth.ToInt32()
    End Function

    Private Function ToolInfoTemplate() As TOOLINFO
        Dim ti As TOOLINFO = New TOOLINFO
        ti.cbSize = Marshal.SizeOf(ti)
        ti.uId = _idHandle
        Return ti
    End Function
End Class
